@@ -2,7 +2,7 @@
 ####**********************************************************************
 ####
 ####  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-####  Version 1.4
+####  Version 1.5.0
 ####
 ####  Copyright 2012, University of Miami
 ####
@@ -64,7 +64,7 @@ plot.rfsrc <- function (x, plots.one.page = TRUE, sorted = TRUE, verbose = TRUE,
 {
   if (sum(inherits(x, c("rfsrc", "grow"), TRUE) == c(1, 2)) != 2 &
       sum(inherits(x, c("rfsrc", "predict"), TRUE) == c(1, 2)) != 2) {
-    stop("This function only works for objects of class `(rfsrc, grow)' or '(rfsrc, predict)'.")
+    stop("this function only works for objects of class `(rfsrc, grow)' or '(rfsrc, predict)'")
   }
   if (is.null(x$err.rate)) {
     stop("object is devoid of performance values")
