@@ -2,7 +2,7 @@
 ////**********************************************************************
 ////
 ////  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-////  Version 1.4
+////  Version 1.5.0
 ////
 ////  Copyright 2012, University of Miami
 ////
@@ -77,7 +77,9 @@ extern uint      RF_totalNodeCount;
 extern int      *RF_seed_;
 extern uint     *RF_tLeafCount_;
 extern double   *RF_proximity_;
+extern double   *RF_weight_;
 extern uint      RF_opt;
+extern uint      RF_optHigh;
 extern uint      RF_splitRule;
 extern uint      RF_splitRandomCount;
 extern uint      RF_nImpute;
@@ -115,13 +117,19 @@ extern uint      RF_timeIndex;
 extern uint      RF_statusIndex;
 extern uint     *RF_yIndex;
 extern uint      RF_ySize;
-extern uint     *RF_testMembershipFlag;  
+extern char     *RF_testMembershipFlag;  
 extern uint      RF_intrPredictorSize;
 extern uint     *RF_intrPredictor;
-extern uint     *RF_intrIndividual;
+extern uint      RF_sobservationSize;
+extern uint     *RF_sobservationIndv;
+extern uint     *RF_gobservationIndv;  
 extern char     *RF_importanceFlag;   
+extern uint      RF_xWeightType;
+extern uint     *RF_xWeightSorted;
+extern uint     *RF_xWeightDensity;
+extern uint      RF_xWeightDensitySize;
 extern uint      RF_eventTypeSize;
-extern uint      RF_mStatusSize; 
+extern uint      RF_mStatusSize;
 extern uint     *RF_eventType;
 extern uint     *RF_eventTypeIndex;
 extern uint     *RF_eIndividualSize;
@@ -155,13 +163,13 @@ extern uint     *RF_fmxFactorIndex;
 extern uint      RF_rMaxFactorLevel;
 extern uint      RF_xMaxFactorLevel;
 extern uint      RF_maxFactorLevel;
-extern char      RF_mStatusFlag; 
-extern char      RF_mTimeFlag; 
-extern char      RF_mResponseFlag; 
-extern char      RF_mPredictorFlag; 
-extern char      RF_fmStatusFlag; 
+extern char      RF_mStatusFlag;
+extern char      RF_mTimeFlag;
+extern char      RF_mResponseFlag;
+extern char      RF_mPredictorFlag;
+extern char      RF_fmStatusFlag;
 extern char      RF_fmTimeFlag;
-extern char      RF_fmResponseFlag; 
+extern char      RF_fmResponseFlag;
 extern char      RF_fmPredictorFlag;
 extern uint     *RF_mRecordMap;
 extern uint     *RF_fmRecordMap;
@@ -175,12 +183,17 @@ extern int     **RF_mpSign;
 extern int     **RF_fmpSign;
 extern int      *RF_mpIndex;
 extern int      *RF_fmpIndex;
+extern double **RF_weightPtr;
 extern double   **RF_importancePtr;
 extern double **RF_sImputeResponsePtr;
 extern double **RF_sImputePredictorPtr;
 extern uint  **RF_tNodeMembershipIndexPtr;
 extern uint  **RF_bootstrapMembershipPtr;
 extern uint  **RF_pNodeMembershipIndexPtr;
+extern double *RF_proximityDen;
+extern uint    RF_rejectedTreeCount;
+extern uint    RF_validTreeCount;
+extern uint    RF_stumpedTreeCount;
 extern double ***RF_oobEnsemblePtr;
 extern double ***RF_fullEnsemblePtr;
 extern double ***RF_oobCIFPtr;
@@ -198,8 +211,8 @@ extern double  ***RF_cVimpEnsemble;
 extern double  ***RF_vimpLeo;
 extern double   **RF_perfLeo;
 extern double ***RF_splitDepthPtr;
-extern uint    *RF_serialTreeIndex;  
-extern uint     RF_serialTreeCount;  
+extern uint    *RF_serialTreeIndex;
+extern uint     RF_serialTreeCount;
 extern char    **RF_dmRecordBootFlag;
 extern double ***RF_dmvImputation;
 extern Terminal ***RF_mTermList;
@@ -207,7 +220,7 @@ extern Terminal ***RF_mTermMembership;
 extern double **RF_performancePtr;
 extern uint   **RF_varUsedPtr;
 extern uint    *RF_oobSize;
-extern uint    *RF_foobSize;
+extern uint    *RF_soobSize;
 extern uint    *RF_tLeafCount;
 extern uint    *RF_nodeCount;
 extern uint    *RF_mwcpCount;
@@ -218,9 +231,10 @@ extern Node   ***RF_tNodeMembership;
 extern Node   ***RF_ftNodeMembership;
 extern Node   ***RF_pNodeMembership;
 extern uint    **RF_bootMembershipIndex;
-extern uint     *RF_trivialBootMembershipIndex;
-extern uint    **RF_bootMembershipFlag;
-extern uint    **RF_oobMembershipFlag;
+extern uint     *RF_identityMembershipIndex;
+extern char    **RF_bootMembershipFlag;
+extern uint    **RF_bootMembershipCount;
+extern char    **RF_oobMembershipFlag;
 extern Node   ***RF_tNodeList;
 extern Node   ***RF_pNodeList;
 extern uint     *RF_orderedLeafCount;
@@ -235,13 +249,13 @@ extern double ***RF_observation;
 extern double ***RF_fobservation;
 extern uint    **RF_masterTimeIndex;
 extern Factor ***RF_factorList;
-extern float (*ran1) (uint);
+extern float (*ran1A) (uint);
 extern void  (*randomSetChain) (uint, int);
 extern int   (*randomGetChain) (uint);
-extern float (*ran2) (uint);
+extern float (*ran1B) (uint);
 extern void  (*randomSetUChain) (uint, int);
 extern int   (*randomGetUChain) (uint);
-extern float (*ran3) (uint);
+extern float (*ran1C) (uint);
 extern void  (*randomSetUChainCov) (uint, int);
 extern int   (*randomGetUChainCov) (uint);
 #endif
