@@ -2,7 +2,7 @@
 ////**********************************************************************
 ////
 ////  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-////  Version 1.4
+////  Version 1.5.0
 ////
 ////  Copyright 2012, University of Miami
 ////
@@ -67,7 +67,7 @@ struct terminal {
   unsigned int nodeID;
   unsigned int *lmiIndex;
   unsigned int  lmiSize;
-  double       *lmiRaggedValue;
+  double       *lmiValue;
   unsigned int  dominant;
 };
 #endif
